program GetBarometer;

{
2012-Jan-23  New program
             Sends three serial characters to the JeeNode board and looks
             for a response.
2012-Feb-15  Store a successful value in the registry, and use it next time
             if we fail to get a reasonable value back.
             Note when a new value is saved

Copyright  David J Taylor, Edinburgh
Web site:  www.satsignal.eu
E-mail:    davidtaylor@writeme.com
}

{$APPTYPE CONSOLE}

uses
  SysUtils,
  CPort,
  CPortTypes,
  Windows,
  SyncObjs,
  RegistryVariables,
  Utils;

var
  Event: TEvent;

function CtrlHandler (CtrlType: LongWord): Boolean;
begin
  Event.SetEvent;
  Result := True;
end;

var
  ComPort: TComPort;
  Events: TComEvents;
  answer, data: AnsiString;
  Step: Integer;
  got_end: boolean;
  index, pressure: integer;
  saved_pressure: TRegistryInteger;
  save_datetime: TRegistryDateTime;
begin
  Event := TEvent.Create (nil, True, False, '');
  SetConsoleCtrlHandler (@CtrlHandler, True);
  saved_pressure := TRegistryInteger.CreateValue ('LastValidPressure', 0);
  save_datetime := TRegistryDateTime.CreateValue ('LastValidTime', 0);

  try
    ComPort := TComPort.Create(nil);
    try
      if ParamCount > 0 then
        ComPort.Port := ParamStr (1)
      else
        ComPort.Port := 'COM1';
      ComPort.Events := [];
      ComPort.BaudRate := br57600;
      ComPort.Open; // open port
      ComPort.SetDTR (False);   // keeps the Reset line high
      ComPort.WriteStr('xxx');
      answer := '';
      Step := 0;
      got_end := False;
      repeat
        Events := [evRxChar];
        ComPort.WaitForEvent (Events, Event.Handle, 5000);  // wait for characters
        if evRxChar in Events then
        begin
          ComPort.ReadStr (data, ComPort.InputCount);
          answer := answer + data;
          index := Pos ('BMP ', answer);
          if index > 0 then
            begin
            answer := Copy (answer, index, 999);
            got_end := Pos (#10, answer) > 26;
            end;
          if got_end then Break;
        end;
        Inc (Step)
      until (Events = []) or (Step = 20);
      index := Pos ('BMP ', answer);
      if index > 0 then
        begin
        answer := Copy (answer, index+4, 999);
        for step := 1 to 3 do
          begin
          index := Pos (' ', Answer);
          answer := Copy (answer, index+1, 999);
          end;
        pressure := StrToInt (Trim (answer));
        if ParamCount > 1 then
          pressure := pressure + StrToInt (ParamStr (2));
        if pressure > 93000 then
          begin
          saved_pressure.Value := pressure; // update the previous value
          save_datetime.Value := NowUTC;    // and note the time
          end;
        end;
    finally
      ComPort.Free;
    end;
  except
  end;

  WriteLn (IntToStr (saved_pressure.Value));
  WriteLn ('0');
  WriteLn ('0');
  WriteLn ('0');

  save_datetime.Free;
  saved_pressure.Free;
  Event.Free;
end.
