// Ports demo, reads out a BMP085 sensor connected via I2C
// 2009-02-17 <jc@wippler.nl> http://opensource.org/licenses/mit-license.php

// 2010-05-22: added support for all resolution modes
// 2010-05-25: extended to also broadcast all readings over wireless
// 2010-06-17: add power saving logic, should reduce consumption by over 90%
// 2010-06-24: improved power savings, several "hot spots" optimized

// see http://news.jeelabs.org/2010/06/20/battery-savings-for-the-pressure-plug/
// see http://news.jeelabs.org/2010/06/30/going-for-gold-with-the-bmp085/

#include <JeeLib.h>
#include <PortsBMP085.h>

PortI2C two (4);
BMP085 psensor (two, 3); // ultra high resolution

// This power-saving code was shamelessly stolen from the rooms.pde sketch,
// see http://code.jeelabs.org/viewvc/svn/jeelabs/trunk/jeemon/sketches/rooms/

void setup() {
    Serial.begin(57600);
    Serial.print("\n[bmp085demo]");
    
    psensor.getCalibData();
}

void loop() {
    // sensor readout takes some time, so go into power down while waiting
    // int32_t traw = psensor.measure(BMP085::TEMP);
    // int32_t praw = psensor.measure(BMP085::PRES);
    
  if (Serial.available()) 
    
  {
    char ch = Serial.read();
    psensor.startMeas(BMP085::TEMP);
    delay(16);
    int32_t traw = psensor.getResult(BMP085::TEMP);

    psensor.startMeas(BMP085::PRES);
    delay(32);
    int32_t praw = psensor.getResult(BMP085::PRES);
    
    struct { int16_t temp; int32_t pres; } payload;
    psensor.calculate(payload.temp, payload.pres);
    
    // this code is not needed for use as remote node, keep it for debugging
    Serial.print("BMP ");
    Serial.print(traw);
    Serial.print(' ');
    Serial.print(praw);    
    Serial.print(' ');
    Serial.print(payload.temp);
    Serial.print(' ');
    Serial.print(payload.pres);
    Serial.print("\n");
    
  }
  delay (300);
}

